@@ -30,6 +30,9 @@ struct _NabiApplication {
     gchar*	    xim_name;
 
     GtkWidget*      palette;
+    GtkWidget*      keyboard_button;
+    GtkWidget*      tray_icon;
+    GtkTooltips*    tooltips;
     gboolean	    status_only;
     gchar*	    session_id;
 
