@@ -95,12 +95,20 @@ static char *nabi_locales[] = {
 };
 
 NabiServer*
-nabi_server_new(const char *name)
+nabi_server_new(Display* display, int screen, const char *name)
 {
+    const char *charset;
+    char *trigger_keys[3]   = { "Hangul", "Shift+space", NULL };
+    char *off_keys[2]       = { "Escape", NULL };
+    char *candidate_keys[3] = { "Hangul_Hanja", "F9", NULL };
+
     NabiServer *server;
 
     server = (NabiServer*)malloc(sizeof(NabiServer));
 
+    server->display = display;
+    server->screen = screen;
+
     /* server var */
     if (name == NULL)
 	server->name = strdup(PACKAGE);
@@ -108,9 +116,6 @@ nabi_server_new(const char *name)
 	server->name = strdup(name);
 
     server->xims = 0;
-    server->widget = NULL;
-    server->root_window = NULL;
-    server->display = NULL;
     server->window = 0;
     server->filter_mask = 0;
     server->trigger_keys.count_keys = 0;
@@ -119,6 +124,25 @@ nabi_server_new(const char *name)
     server->off_keys.keylist = NULL;
     server->candidate_keys.count_keys = 0;
     server->candidate_keys.keylist = NULL;
+    server->locales = nabi_locales;
+    server->filter_mask = KeyPressMask;
+
+    /* keys */
+    nabi_server_set_trigger_keys(server, trigger_keys);
+    nabi_server_set_off_keys(server, off_keys);
+    nabi_server_set_candidate_keys(server, candidate_keys);
+
+    /* check locale encoding */
+    if (g_get_charset(&charset)) {
+	/* utf8 encoding */
+	/* We add current locale to nabi support  locales array,
+	 * so whatever the locale string is, if its encoding is utf8,
+	 * nabi support the locale */
+	int i;
+	for (i = 0; server->locales[i] != NULL; i++)
+	    continue;
+	server->locales[i] = setlocale(LC_CTYPE, NULL);
+    }
 
     /* connection list */
     server->connections = NULL;
@@ -137,12 +161,13 @@ nabi_server_new(const char *name)
     server->auto_reorder = True;
     server->input_mode = NABI_INPUT_MODE_DIRECT;
     server->input_mode_option = NABI_INPUT_MODE_PER_TOPLEVEL;
+    server->output_mode = NABI_OUTPUT_SYLLABLE;
 
     /* hanja */
-    server->hanja_table = NULL;
+    server->hanja_table = hanja_table_load(NULL);
 
     /* symbol */
-    server->symbol_table = NULL;
+    server->symbol_table = hanja_table_load(NABI_SYMBOL_TABLE);
 
     /* options */
     server->show_status = False;
@@ -155,7 +180,8 @@ nabi_server_new(const char *name)
     server->preedit_bg.red = 0;
     server->preedit_bg.green = 0;
     server->preedit_bg.blue = 0;
-    server->candidate_font = NULL;
+    server->preedit_font = pango_font_description_from_string("Sans 9");
+    server->candidate_font = pango_font_description_from_string("Sans 14");
 
     /* statistics */
     memset(&(server->statistics), 0, sizeof(server->statistics));
@@ -171,9 +197,6 @@ nabi_server_destroy(NabiServer *server)
     if (server == NULL)
 	return;
 
-    if (server->gc != NULL)
-	g_object_unref(G_OBJECT(server->gc));
-
     /* destroy remaining connections */
     if (server->connections != NULL) {
 	item = server->connections;
@@ -221,7 +244,8 @@ nabi_server_destroy(NabiServer *server)
     g_free(server->candidate_keys.keylist);
     g_free(server->off_keys.keylist);
 
-    g_free(server->candidate_font);
+    pango_font_description_free(server->preedit_font);
+    pango_font_description_free(server->candidate_font);
     g_free(server->name);
 
     g_free(server);
@@ -242,17 +266,18 @@ nabi_server_set_hangul_keyboard(NabiServer *server, const char *id)
 void
 nabi_server_set_mode_info(NabiServer *server, int state)
 {
-    guint32 data;
+    long data;
 
     if (server == NULL)
 	return;
 
     data = state;
-    gdk_property_change(server->root_window,
-			gdk_atom_intern ("_HANGUL_INPUT_MODE", FALSE),
-			gdk_atom_intern ("INTEGER", FALSE),
-			32, GDK_PROP_MODE_REPLACE,
-			(const guchar *)&data, 1);
+    Window root = RootWindow(server->display, server->screen);
+    Atom property = XInternAtom(server->display, "_HANGUL_INPUT_MODE", FALSE);
+    Atom type = XInternAtom(server->display, "INTEGER", FALSE);
+
+    XChangeProperty(server->display, root, property, type, 
+		    32, PropModeReplace, (unsigned char*)&data, 1);
 }
 
 void
@@ -269,15 +294,27 @@ nabi_server_set_output_mode(NabiServer *server, NabiOutputMode mode)
 }
 
 void
-nabi_server_set_candidate_font(NabiServer *server, const gchar *font_name)
+nabi_server_set_preedit_font(NabiServer *server, const char *font_desc)
 {
     if (server == NULL)
 	return;
 
-    if (font_name != NULL) {
-	g_free(server->candidate_font);
-	server->candidate_font = g_strdup(font_name);
-    }
+    if (font_desc != NULL)
+	pango_font_description_free(server->preedit_font);
+    server->preedit_font = pango_font_description_from_string(font_desc);
+    nabi_log(3, "set preedit font: %s\n", font_desc);
+}
+
+void
+nabi_server_set_candidate_font(NabiServer *server, const char *font_desc)
+{
+    if (server == NULL)
+	return;
+
+    if (font_desc != NULL)
+	pango_font_description_free(server->candidate_font);
+    server->candidate_font = pango_font_description_from_string(font_desc);
+    nabi_log(3, "set candidate font: %s\n", font_desc);
 }
 
 static void
@@ -359,42 +396,6 @@ nabi_server_set_candidate_keys(NabiServer *server, char **keys)
     xim_trigger_keys_set_value(&server->candidate_keys, keys);
 }
 
-void
-nabi_server_init(NabiServer *server)
-{
-    const char *charset;
-    char *trigger_keys[3]   = { "Hangul", "Shift+space", NULL };
-    char *off_keys[2]       = { "Escape", NULL };
-    char *candidate_keys[3] = { "Hangul_Hanja", "F9", NULL };
-
-    if (server == NULL)
-	return;
-
-    server->locales = nabi_locales;
-    server->filter_mask = KeyPressMask;
-
-    nabi_server_set_trigger_keys(server, trigger_keys);
-    nabi_server_set_off_keys(server, off_keys);
-    nabi_server_set_candidate_keys(server, candidate_keys);
-
-    server->output_mode = NABI_OUTPUT_SYLLABLE;
-
-    /* check locale encoding */
-    if (g_get_charset(&charset)) {
-	/* utf8 encoding */
-	/* We add current locale to nabi support  locales array,
-	 * so whatever the locale string is, if its encoding is utf8,
-	 * nabi support the locale */
-	int i;
-	for (i = 0; server->locales[i] != NULL; i++)
-	    continue;
-	server->locales[i] = setlocale(LC_CTYPE, NULL);
-    }
-
-    server->hanja_table = hanja_table_load(NULL);
-    server->symbol_table = hanja_table_load(NABI_SYMBOL_TABLE);
-}
-
 gboolean
 nabi_server_is_valid_ic(NabiServer* server, NabiIC* ic)
 {
@@ -484,9 +485,8 @@ nabi_server_is_running(const char* name)
 }
 
 int
-nabi_server_start(NabiServer *server, GtkWidget *widget)
+nabi_server_start(NabiServer *server)
 {
-    Display *display;
     Window window;
     XIMS xims;
     XIMStyles input_styles;
@@ -499,8 +499,9 @@ nabi_server_start(NabiServer *server, GtkWidget *widget)
     if (server->xims != NULL)
 	return 0;
 
-    display = GDK_WINDOW_XDISPLAY(widget->window);
-    window = GDK_WINDOW_XWINDOW(widget->window);
+    window = XCreateSimpleWindow(server->display,
+				 RootWindow(server->display, server->screen),
+				 0, 0, 1, 1, 1, 0, 0);
 
     input_styles.count_styles = sizeof(nabi_input_styles) 
 		    / sizeof(XIMStyle) - 1;
@@ -511,7 +512,7 @@ nabi_server_start(NabiServer *server, GtkWidget *widget)
     encodings.supported_encodings = nabi_encodings;
 
     locales = g_strjoinv(",", server->locales);
-    xims = IMOpenIM(display,
+    xims = IMOpenIM(server->display,
 		   IMModifiers, "Xi18n",
 		   IMServerWindow, window,
 		   IMServerName, server->name,
@@ -539,15 +540,8 @@ nabi_server_start(NabiServer *server, GtkWidget *widget)
 		  NULL);
 
     server->xims = xims;
-    server->widget = widget;
-    server->root_window = gdk_screen_get_root_window(gtk_widget_get_screen(widget));
-    server->display = display;
     server->window = window;
 
-    server->gc = gdk_gc_new(widget->window);
-    gdk_gc_set_foreground(server->gc, &(server->preedit_fg));
-    gdk_gc_set_background(server->gc, &(server->preedit_bg));
-
     server->start_time = time(NULL);
 
     nabi_log(1, "xim server started\n");
@@ -863,6 +857,12 @@ nabi_server_get_keyboard_name_by_id(NabiServer* server, const char* id)
     return NULL;
 }
 
+const char*
+nabi_server_get_current_keyboard_name(NabiServer* server)
+{
+    return nabi_server_get_keyboard_name_by_id(server, server->hangul_keyboard);
+}
+
 KeySym
 nabi_server_normalize_keysym(NabiServer *server,
 			     KeySym keysym, unsigned int state)
