@@ -83,18 +83,16 @@ struct NabiStatistics {
 
 struct _NabiServer {
     /* XIMS */
+    Display*                display;
+    int                     screen;
     char*		    name;
     XIMS                    xims;
-    GtkWidget*              widget;
-    GdkWindow*              root_window;
-    Display*                display;
     Window                  window;
     long                    filter_mask;
     XIMTriggerKeys          trigger_keys;
     XIMTriggerKeys          off_keys;
     XIMTriggerKeys          candidate_keys;
     char**                  locales;
-    GdkGC*		    gc;
 
     /* xim connection list */
     GSList*                 connections;
@@ -126,7 +124,9 @@ struct _NabiServer {
     NabiInputModeOption     input_mode_option;
     GdkColor                preedit_fg;
     GdkColor                preedit_bg;
-    gchar*		    candidate_font;
+
+    PangoFontDescription*   preedit_font;
+    PangoFontDescription*   candidate_font;
 
     /* statistics */
     time_t                  start_time;
@@ -135,11 +135,11 @@ struct _NabiServer {
 
 extern NabiServer* nabi_server;
 
-NabiServer* nabi_server_new		(const char *name);
+NabiServer* nabi_server_new		(Display*    display,
+					 int         screen,
+					 const char* name);
 void        nabi_server_destroy         (NabiServer* server);
-void        nabi_server_init            (NabiServer* server);
-int         nabi_server_start           (NabiServer* server,
-					 GtkWidget*  widget);
+int         nabi_server_start           (NabiServer* server);
 int         nabi_server_stop            (NabiServer *server);
 
 Bool        nabi_server_is_running();
@@ -167,8 +167,10 @@ void        nabi_server_toggle_input_mode(NabiServer* server);
 void        nabi_server_set_mode_info(NabiServer *server, int state);
 void        nabi_server_set_output_mode (NabiServer *server,
 					 NabiOutputMode mode);
+void	    nabi_server_set_preedit_font(NabiServer *server,
+					   const gchar *font_desc);
 void	    nabi_server_set_candidate_font(NabiServer *server,
-					   const gchar *font_name);
+					   const gchar *font_desc);
 void        nabi_server_set_dynamic_event_flow(NabiServer* server, Bool flag);
 void        nabi_server_set_xim_name(NabiServer* server, const char* name);
 void        nabi_server_set_commit_by_word(NabiServer* server, Bool flag);
@@ -211,8 +213,9 @@ Bool	    nabi_server_load_compose_table(NabiServer *server,
 const NabiHangulKeyboard*
 nabi_server_get_hangul_keyboard_list(NabiServer* server);
 
-const char*
-nabi_server_get_keyboard_name_by_id(NabiServer* server, const char* id);
+const char* nabi_server_get_keyboard_name_by_id(NabiServer* server,
+                                                const char* id);
+const char* nabi_server_get_current_keyboard_name(NabiServer* server);
 
 #endif  /* __SERVER_H_ */
 
