@@ -79,6 +79,8 @@ static GtkWidget *about_dialog = NULL;
 static GtkWidget *preference_dialog = NULL;
 
 static void nabi_app_load_base_icons();
+static void nabi_app_update_tooltips();
+static void nabi_app_update_keyboard_button();
 
 static void nabi_state_icon_load(NabiStateIcon* state, int w, int h);
 
@@ -356,6 +358,8 @@ nabi_app_setup_server(void)
 	nabi_server_set_input_mode_option(nabi_server,
 					  NABI_INPUT_MODE_PER_TOPLEVEL);
     }
+
+    nabi_server_set_preedit_font(nabi_server, nabi->config->preedit_font);
 }
 
 void
@@ -528,12 +532,15 @@ nabi_menu_position_func(GtkMenu *menu,
 			gboolean *push_in,
 			gpointer data)
 {
+    GtkWidget *widget;
     GdkWindow *window;
     GdkScreen *screen;
     gint width, height, menu_width, menu_height;
     gint screen_width, screen_height;
 
-    window = GDK_WINDOW(GTK_WIDGET(data)->window);
+    widget = GTK_WIDGET(data);
+
+    window = GDK_WINDOW(widget->window);
     screen = gdk_drawable_get_screen(GDK_DRAWABLE(window));
     screen_width = gdk_screen_get_width(screen);
     screen_height = gdk_screen_get_height(screen);
@@ -545,10 +552,12 @@ nabi_menu_position_func(GtkMenu *menu,
 			  &menu_width, &menu_height);
 
     if (*y + height < screen_height / 2)
-	*y += height;
+	*y += widget->allocation.height;
     else
 	*y -= menu_height;
 
+    *x += widget->allocation.x;
+    *y += widget->allocation.y;
     if (*x + menu_width > screen_width)
 	*x = screen_width - menu_width;
 }
@@ -868,23 +877,14 @@ on_menu_hide_palette(GtkWidget *widget, gpointer data)
 static void
 on_menu_keyboard(GtkWidget *widget, gpointer data)
 {
-    GtkWidget* title_menu;
     const char* id = (const char*)data;
 
     nabi_app_set_hangul_keyboard(id);
     preference_window_update();
     nabi_app_save_config();
 
-    title_menu = g_object_get_data(G_OBJECT(widget), "nabi_title_menu");
-    if (title_menu != NULL) {
-	GtkWidget* label;
-	label = gtk_bin_get_child(GTK_BIN(title_menu));
-	if (label != NULL) {
-	    const char* name;
-	    name = nabi_server_get_keyboard_name_by_id(nabi_server, id);
-	    gtk_label_set_text(GTK_LABEL(label), _(name));
-	}
-    }
+    nabi_app_update_keyboard_button();
+    nabi_app_update_tooltips();
 }
 
 static void
@@ -1136,8 +1136,6 @@ nabi_create_tray_icon(gpointer data)
     GtkWidget *eventbox;
     GtkTooltips *tooltips;
     GtkOrientation orientation;
-    const char *keyboard_name;
-    char tip_text[256];
     int w, h;
 
     if (nabi_tray != NULL)
@@ -1155,16 +1153,11 @@ nabi_create_tray_icon(gpointer data)
     g_signal_connect(G_OBJECT(eventbox), "button-press-event",
 		     G_CALLBACK(on_tray_icon_button_press), NULL);
     gtk_widget_show(eventbox);
-
-    keyboard_name = nabi_server_get_keyboard_name_by_id(nabi_server,
-					nabi->config->hangul_keyboard);
-    snprintf(tip_text, sizeof(tip_text), _("Nabi: %s"), _(keyboard_name));
+    nabi->tray_icon = eventbox;
 
     tooltips = gtk_tooltips_new();
-    gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), eventbox,
-			 tip_text,
-			 _("Hangul input method: Nabi"
-			   " - You can input hangul using this program"));
+    nabi->tooltips = tooltips;
+    nabi_app_update_tooltips();
 
     orientation = egg_tray_icon_get_orientation(tray->widget);
     if (orientation == GTK_ORIENTATION_VERTICAL) {
@@ -1190,17 +1183,24 @@ nabi_create_tray_icon(gpointer data)
     return FALSE;
 }
 
+static void
+on_palette_item_pressed(GtkWidget* widget, gpointer data)
+{
+    gtk_menu_popup(GTK_MENU(data), NULL, NULL,
+	    nabi_menu_position_func, widget,
+	    0, gtk_get_current_event_time());
+}
+
 GtkWidget*
 nabi_app_create_palette(void)
 {
     GtkWidget* handlebox;
     GtkWidget* hbox;
     GtkWidget* eventbox;
-    GtkWidget* menubar;
-    GtkWidget* submenu = NULL;
+    GtkWidget* menu;
     GtkWidget* menuitem;
-    GtkWidget* title_menuitem;
     GtkWidget* image;
+    GtkWidget* button;
     const char* current_keyboard_name = NULL;
 
     nabi_palette = g_new(NabiPalette, 1);
@@ -1230,71 +1230,77 @@ nabi_app_create_palette(void)
     gtk_container_add(GTK_CONTAINER(eventbox), nabi_palette->state->widget);
     gtk_widget_show(nabi_palette->state->widget);
 
-    menubar = gtk_menu_bar_new();
-    gtk_box_pack_start(GTK_BOX(hbox), menubar, FALSE, FALSE, 0);
-
     current_keyboard_name = nabi_server_get_keyboard_name_by_id(nabi_server,
 				    nabi->config->hangul_keyboard);
     if (current_keyboard_name != NULL) {
 	const NabiHangulKeyboard* keyboards;
 
-	menuitem = gtk_menu_item_new_with_label(_(current_keyboard_name));
-	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), menuitem);
-	title_menuitem = menuitem;
+	button = gtk_button_new_with_label(_(current_keyboard_name));
+	gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+	gtk_button_set_focus_on_click(GTK_BUTTON(button), FALSE);
+	GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+	gtk_widget_show(button);
+	nabi->keyboard_button = button;
 
 	keyboards = nabi_server_get_hangul_keyboard_list(nabi_server);
 	if (keyboards != NULL) {
 	    int i;
-	    submenu = gtk_menu_new();
-	    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
-
+	    menu = gtk_menu_new();
 	    for (i = 0; keyboards[i].id != NULL; i++) {
 		menuitem = gtk_menu_item_new_with_label(_(keyboards[i].name));
-		gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
-		g_object_set_data(G_OBJECT(menuitem),
-				  "nabi_title_menu", title_menuitem);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+		gtk_widget_show(menuitem);
 		g_signal_connect(G_OBJECT(menuitem), "activate",
 				 G_CALLBACK(on_menu_keyboard),
 				 (gpointer)keyboards[i].id);
 	    }
+
+	    g_signal_connect(G_OBJECT(button), "pressed",
+			     G_CALLBACK(on_palette_item_pressed), menu);
 	}
     }
 
     image = gtk_image_new_from_stock(GTK_STOCK_PROPERTIES, GTK_ICON_SIZE_MENU);
-    menuitem = gtk_image_menu_item_new();
-    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
-    gtk_menu_shell_append(GTK_MENU_SHELL(menubar), menuitem);
+    button = gtk_button_new();
+    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+    gtk_button_set_focus_on_click(GTK_BUTTON(button), FALSE);
+    GTK_WIDGET_UNSET_FLAGS(button, GTK_CAN_FOCUS);
+    gtk_button_set_image(GTK_BUTTON(button), image);
+    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    gtk_widget_show(button);
 
-    submenu = gtk_menu_new();
-    gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuitem), submenu);
+    menu = gtk_menu_new();
 
     menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_DIALOG_INFO, NULL);
-    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
     g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
 			     G_CALLBACK(on_menu_about), menuitem);
 
     menuitem = gtk_separator_menu_item_new();
-    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
     menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
-    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
     g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
 			     G_CALLBACK(on_menu_preference), menuitem);
 
     menuitem = gtk_menu_item_new_with_mnemonic(_("_Hide palette"));
-    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
     g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
 			     G_CALLBACK(on_menu_hide_palette), menuitem);
 
     menuitem = gtk_separator_menu_item_new();
-    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
     menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
-    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menuitem);
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
     g_signal_connect_swapped(G_OBJECT(menuitem), "activate",
 			     G_CALLBACK(on_menu_quit), menuitem);
 
-    gtk_widget_show_all(menubar);
+    gtk_widget_show_all(menu);
+    g_signal_connect(G_OBJECT(button), "pressed",
+		     G_CALLBACK(on_palette_item_pressed), menu);
 
     g_idle_add(nabi_create_tray_icon, NULL);
 
@@ -1388,4 +1394,31 @@ nabi_app_save_config()
 	nabi_config_save(nabi->config);
 }
 
+static void
+nabi_app_update_keyboard_button()
+{
+    if (nabi->keyboard_button != NULL) {
+	const char* name;
+	name = nabi_server_get_current_keyboard_name(nabi_server);
+	gtk_button_set_label(GTK_BUTTON(nabi->keyboard_button), _(name));
+    }
+}
+
+static void
+nabi_app_update_tooltips()
+{
+    if (nabi->tooltips != NULL) {
+	const char* keyboard_name;
+	char tip_text[256];
+	keyboard_name = nabi_server_get_keyboard_name_by_id(nabi_server,
+					    nabi->config->hangul_keyboard);
+	snprintf(tip_text, sizeof(tip_text), _("Nabi: %s"), _(keyboard_name));
+
+	gtk_tooltips_set_tip(GTK_TOOLTIPS(nabi->tooltips), nabi->tray_icon,
+			     tip_text,
+			     _("Hangul input method: Nabi"
+			       " - You can input hangul using this program"));
+    }
+}
+
 /* vim: set ts=8 sts=4 sw=4 : */
